@@ -43,7 +43,12 @@ public class ProjectHelper {
     /** The URI for antlib current definitions */
     public static final String ANT_CURRENT_URI      = "ant:current";
 
-    /** The URI for defined types/tasks - the format is antlib:<package> */
+    /** The URI for ant specific attributes
+     * @since Ant 1.9.1
+     * */
+    public static final String ANT_ATTRIBUTE_URI      = "ant:attribute";
+
+    /** The URI for defined types/tasks - the format is antlib:&lt;package&gt; */
     public static final String ANTLIB_URI     = "antlib:";
 
     /** Polymorphic attribute  */
@@ -66,6 +71,12 @@ public class ProjectHelper {
      */
     public static final String PROJECTHELPER_REFERENCE = MagicNames.REFID_PROJECT_HELPER;
 
+    /**
+     * constant to denote use project name as target prefix
+     * @since Ant 1.9.1
+     */
+    public static final String USE_PROJECT_NAME_AS_TARGET_PREFIX = "USE_PROJECT_NAME_AS_TARGET_PREFIX";
+
     /**
      * Configures the project with the contents of the specified build file.
      *
@@ -82,16 +93,69 @@ public class ProjectHelper {
         helper.parse(project, buildFile);
     }
 
+    /**
+     * Possible value for target's onMissingExtensionPoint attribute. It determines how to deal with
+     * targets that want to extend missing extension-points.
+     * <p>
+     * This class behaves like a Java 1.5 Enum class.
+     *
+     * @since 1.8.2
+     */
+    public static final class OnMissingExtensionPoint {
+
+        /** fail if the extension-point is not defined */
+        public static final OnMissingExtensionPoint FAIL = new OnMissingExtensionPoint(
+                "fail");
+
+        /** warn if the extension-point is not defined */
+        public static final OnMissingExtensionPoint WARN = new OnMissingExtensionPoint(
+                "warn");
+
+        /** ignore the extensionOf attribute if the extension-point is not defined */
+        public static final OnMissingExtensionPoint IGNORE = new OnMissingExtensionPoint(
+                "ignore");
+
+        private static final OnMissingExtensionPoint[] values = new OnMissingExtensionPoint[] {
+                                FAIL, WARN, IGNORE };
+
+        private final String name;
+
+        private OnMissingExtensionPoint(String name) {
+            this.name = name;
+        }
+
+        public String name() {
+            return name;
+        }
+
+        public String toString() {
+            return name;
+        }
+
+        public static OnMissingExtensionPoint valueOf(String name) {
+            if (name == null) {
+                throw new NullPointerException();
+            }
+            for (OnMissingExtensionPoint value : values) {
+                if (name.equals(value.name())) {
+                    return value;
+                }
+            }
+            throw new IllegalArgumentException(
+                    "Unknown onMissingExtensionPoint " + name);
+        }
+    }
+
     /** Default constructor */
     public ProjectHelper() {
     }
 
     // -------------------- Common properties  --------------------
-    // The following properties are required by import ( and other tasks
-    // that read build files using ProjectHelper ).
+    // The following properties are required by import (and other tasks
+    // that read build files using ProjectHelper).
 
-    private Vector importStack = new Vector();
-    private List extensionStack = new LinkedList();
+    private Vector<Object> importStack = new Vector<Object>();
+    private List<String[]> extensionStack = new LinkedList<String[]>();
 
     /**
      *  Import stack.
@@ -100,7 +164,7 @@ public class ProjectHelper {
      *
      * @return the stack of import source objects.
      */
-    public Vector getImportStack() {
+    public Vector<Object> getImportStack() {
         return importStack;
     }
 
@@ -108,19 +172,16 @@ public class ProjectHelper {
      * Extension stack.
      * Used to keep track of targets that extend extension points.
      *
-     * @return a list of two element string arrays where the first
-     * element is the name of the extensionpoint and the second the
-     * name of the target
+     * @return a list of three element string arrays where the first
+     * element is the name of the extensionpoint, the second the name
+     * of the target and the third the name of the enum like class
+     * {@link OnMissingExtensionPoint}.
      */
-    public List getExtensionStack() {
+    public List<String[]> getExtensionStack() {
         return extensionStack;
     }
 
-    private final static ThreadLocal targetPrefix = new ThreadLocal() {
-            protected Object initialValue() {
-                return (String) null;
-            }
-        };
+    private static final ThreadLocal<String> targetPrefix = new ThreadLocal<String>();
 
     /**
      * The prefix to prepend to imported target names.
@@ -132,20 +193,21 @@ public class ProjectHelper {
      * @since Ant 1.8.0
      */
     public static String getCurrentTargetPrefix() {
-        return (String) targetPrefix.get();
+        return targetPrefix.get();
     }
 
     /**
      * Sets the prefix to prepend to imported target names.
      *
+     * @param prefix String
      * @since Ant 1.8.0
      */
     public static void setCurrentTargetPrefix(String prefix) {
         targetPrefix.set(prefix);
     }
 
-    private final static ThreadLocal prefixSeparator = new ThreadLocal() {
-            protected Object initialValue() {
+    private static final ThreadLocal<String> prefixSeparator = new ThreadLocal<String>() {
+            protected String initialValue() {
                 return ".";
             }
         };
@@ -153,25 +215,27 @@ public class ProjectHelper {
     /**
      * The separator between the prefix and the target name.
      *
-     * <p>May be set by &lt;import&gt;'s prefixSeperator attribute.</p>
+     * <p>May be set by &lt;import&gt;'s prefixSeparator attribute.</p>
      *
+     * @return String
      * @since Ant 1.8.0
      */
     public static String getCurrentPrefixSeparator() {
-        return (String) prefixSeparator.get();
+        return prefixSeparator.get();
     }
 
     /**
      * Sets the separator between the prefix and the target name.
      *
+     * @param sep String
      * @since Ant 1.8.0
      */
     public static void setCurrentPrefixSeparator(String sep) {
         prefixSeparator.set(sep);
     }
 
-    private final static ThreadLocal inIncludeMode = new ThreadLocal() {
-            protected Object initialValue() {
+    private static final ThreadLocal<Boolean> inIncludeMode = new ThreadLocal<Boolean>() {
+            protected Boolean initialValue() {
                 return Boolean.FALSE;
             }
         };
@@ -189,20 +253,22 @@ public class ProjectHelper {
      * overwritten in the importing build file.  The depends list of
      * the imported targets is not modified at all.</p>
      *
+     * @return boolean
      * @since Ant 1.8.0
      */
     public static boolean isInIncludeMode() {
-        return inIncludeMode.get() == Boolean.TRUE;
+        return Boolean.TRUE.equals(inIncludeMode.get());
     }
 
     /**
      * Sets whether the current file should be read in include as
      * opposed to import mode.
      *
+     * @param includeMode boolean
      * @since Ant 1.8.0
      */
     public static void setInIncludeMode(boolean includeMode) {
-        inIncludeMode.set(includeMode ? Boolean.TRUE : Boolean.FALSE);
+        inIncludeMode.set(Boolean.valueOf(includeMode));
     }
 
     // --------------------  Parse method  --------------------
@@ -226,7 +292,7 @@ public class ProjectHelper {
 
     /**
      * Get the first project helper found in the classpath
-     * 
+     *
      * @return an project helper, never <code>null</code>
      * @see org.apache.tools.ant.ProjectHelperRepository#getHelpers()
      */
@@ -235,7 +301,7 @@ public class ProjectHelper {
     }
 
     /**
-     * JDK1.1 compatible access to the context class loader. Cut & paste from JAXP.
+     * JDK1.1 compatible access to the context class loader. Cut &amp; paste from JAXP.
      *
      * @deprecated since 1.6.x.
      *             Use LoaderUtils.getContextClassLoader()
@@ -243,6 +309,7 @@ public class ProjectHelper {
      * @return the current context class loader, or <code>null</code>
      * if the context class loader is unavailable.
      */
+    @Deprecated
     public static ClassLoader getContextClassLoader() {
         return LoaderUtils.isContextLoaderAvailable() ? LoaderUtils.getContextClassLoader() : null;
     }
@@ -265,6 +332,7 @@ public class ProjectHelper {
      * @exception BuildException if any of the attributes can't be handled by
      *                           the target
      */
+    @Deprecated
     public static void configure(Object target, AttributeList attrs,
                                  Project project) throws BuildException {
         if (target instanceof TypeAdapter) {
@@ -368,6 +436,7 @@ public class ProjectHelper {
      *             Use project.replaceProperties().
      * @since 1.5
      */
+    @Deprecated
      public static String replaceProperties(Project project, String value) throws BuildException {
         // needed since project properties are not accessible
          return project.replaceProperties(value);
@@ -382,7 +451,7 @@ public class ProjectHelper {
      * @param value The string to be scanned for property references.
      *              May be <code>null</code>, in which case this
      *              method returns immediately with no effect.
-     * @param keys  Mapping (String to String) of property names to their
+     * @param keys  Mapping (String to Object) of property names to their
      *              values. Must not be <code>null</code>.
      *
      * @exception BuildException if the string contains an opening
@@ -393,7 +462,8 @@ public class ProjectHelper {
      * @deprecated since 1.6.x.
      *             Use PropertyHelper.
      */
-     public static String replaceProperties(Project project, String value, Hashtable keys)
+    @Deprecated
+     public static String replaceProperties(Project project, String value, Hashtable<String, Object> keys)
              throws BuildException {
         PropertyHelper ph = PropertyHelper.getPropertyHelper(project);
         return ph.replaceProperties(null, value, keys);
@@ -420,7 +490,8 @@ public class ProjectHelper {
      * @exception BuildException if the string contains an opening
      *                           <code>${</code> without a closing <code>}</code>
      */
-    public static void parsePropertyString(String value, Vector fragments, Vector propertyRefs)
+    @Deprecated
+    public static void parsePropertyString(String value, Vector<String> fragments, Vector<String> propertyRefs)
             throws BuildException {
         PropertyHelper.parsePropertyStringDefault(value, fragments, propertyRefs);
     }
@@ -430,7 +501,7 @@ public class ProjectHelper {
      * For BC purposes the names from the ant core uri will be
      * mapped to "name", other names will be mapped to
      * uri + ":" + name.
-     * @param uri   The namepace URI
+     * @param uri   The namespace URI
      * @param name  The localname
      * @return      The stringified form of the ns name
      */
@@ -472,6 +543,16 @@ public class ProjectHelper {
         return componentName.substring(index + 1);
     }
 
+    /**
+     * Convert an attribute namespace to a "component name".
+     * @param ns the xml namespace uri.
+     * @return the converted value.
+     * @since Ant 1.9.1
+     */
+    public static String nsToComponentName(String ns) {
+        return "attribute namespace:" + ns;
+    }
+
     /**
      * Add location to build exception.
      * @param ex the build exception, if the build exception
@@ -491,6 +572,13 @@ public class ProjectHelper {
             + System.getProperty("line.separator")
             + ex.getLocation().toString()
             + ex.getMessage();
+        if (ex instanceof ExitStatusException) {
+            int exitStatus = ((ExitStatusException) ex).getStatus();
+            if (newLocation == null) {
+                return new ExitStatusException(errorMessage, exitStatus);
+            }
+            return new ExitStatusException(errorMessage, exitStatus, newLocation);
+        }
         if (newLocation == null) {
             return new BuildException(errorMessage, ex);
         }
@@ -513,6 +601,8 @@ public class ProjectHelper {
      *
      * <p>This implementation returns false.</p>
      *
+     * @param r Resource
+     * @return boolean
      * @since Ant 1.8.0
      */
     public boolean canParseAntlibDescriptor(Resource r) {
@@ -523,6 +613,9 @@ public class ProjectHelper {
      * Parse the given URL as an antlib descriptor and return the
      * content as something that can be turned into an Antlib task.
      *
+     * @param containingProject Project
+     * @param source Resource
+     * @return UnknownElement
      * @since ant 1.8.0
      */
     public UnknownElement parseAntlibDescriptor(Project containingProject,
@@ -533,7 +626,7 @@ public class ProjectHelper {
     /**
      * Check if the helper supports the kind of file. Some basic check on the
      * extension's file should be done here.
-     * 
+     *
      * @param buildFile
      *            the file expected to be parsed (never <code>null</code>)
      * @return true if the helper supports it
@@ -545,11 +638,83 @@ public class ProjectHelper {
 
     /**
      * The file name of the build script to be parsed if none specified on the command line
-     * 
+     *
      * @return the name of the default file (never <code>null</code>)
      * @since Ant 1.8.0
      */
     public String getDefaultBuildFile() {
         return Main.DEFAULT_BUILD_FILENAME;
     }
+
+    /**
+     * Check extensionStack and inject all targets having extensionOf attributes
+     * into extensionPoint.
+     * <p>
+     * This method allow you to defer injection and have a powerful control of
+     * extensionPoint wiring.
+     * </p>
+     * <p>
+     * This should be invoked by each concrete implementation of ProjectHelper
+     * when the root "buildfile" and all imported/included buildfile are loaded.
+     * </p>
+     *
+     * @param project The project containing the target. Must not be
+     *            <code>null</code>.
+     * @exception BuildException if OnMissingExtensionPoint.FAIL and
+     *                extensionPoint does not exist
+     * @see OnMissingExtensionPoint
+     * @since 1.9
+     */
+    public void resolveExtensionOfAttributes(Project project)
+            throws BuildException {
+        for (String[] extensionInfo : getExtensionStack()) {
+            String extPointName = extensionInfo[0];
+            String targetName = extensionInfo[1];
+            OnMissingExtensionPoint missingBehaviour = OnMissingExtensionPoint.valueOf(extensionInfo[2]);
+            // if the file has been included or imported, it may have a prefix
+            // we should consider when trying to resolve the target it is
+            // extending
+            String prefixAndSep = extensionInfo.length > 3 ? extensionInfo[3] : null;
+
+            // find the target we're extending
+            Hashtable<String, Target> projectTargets = project.getTargets();
+            Target extPoint = null;
+            if (prefixAndSep == null) {
+                // no prefix - not from an imported/included build file
+                extPoint = projectTargets.get(extPointName);
+            } else {
+                // we have a prefix, which means we came from an include/import
+
+                // FIXME: here we handle no particular level of include. We try
+                // the fully prefixed name, and then the non-prefixed name. But
+                // there might be intermediate project in the import stack,
+                // which prefix should be tested before testing the non-prefix
+                // root name.
+
+                extPoint = projectTargets.get(prefixAndSep + extPointName);
+                if (extPoint == null) {
+                    extPoint = projectTargets.get(extPointName);
+                }
+            }
+
+            // make sure we found a point to extend on
+            if (extPoint == null) {
+                String message = "can't add target " + targetName
+                        + " to extension-point " + extPointName
+                        + " because the extension-point is unknown.";
+                if (missingBehaviour == OnMissingExtensionPoint.FAIL) {
+                    throw new BuildException(message);
+                } else if (missingBehaviour == OnMissingExtensionPoint.WARN) {
+                    Target t = projectTargets.get(targetName);
+                    project.log(t, "Warning: " + message, Project.MSG_WARN);
+                }
+            } else {
+                if (!(extPoint instanceof ExtensionPoint)) {
+                    throw new BuildException("referenced target " + extPointName
+                            + " is not an extension-point");
+                }
+                extPoint.addDependency(targetName);
+            }
+        }
+    }
 }
