@@ -253,12 +253,11 @@ public class BorlandDeploymentTool extends GenericDeploymentTool
         File borlandDD = new File(getConfig().descriptorDir, ddPrefix + dd);
         if (borlandDD.exists()) {
             log("Borland specific file found " + borlandDD,  Project.MSG_VERBOSE);
-            ejbFiles.put(META_DIR + dd ,  borlandDD);
+            ejbFiles.put(META_DIR + dd, borlandDD);
         } else {
             log("Unable to locate borland deployment descriptor. "
                 + "It was expected to be in "
                 + borlandDD.getPath(), Project.MSG_WARN);
-            return;
         }
     }
 
@@ -326,7 +325,7 @@ public class BorlandDeploymentTool extends GenericDeploymentTool
      * @param sourceJar java.io.File representing the produced jar file
      */
     private void verifyBorlandJarV4(File sourceJar) {
-        org.apache.tools.ant.taskdefs.Java javaTask = null;
+        Java javaTask = null;
         log("verify BAS " + sourceJar, Project.MSG_INFO);
         try  {
             String args = verifyArgs;
