@@ -24,12 +24,14 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
 import java.net.URLConnection;
+
 import javax.swing.ImageIcon;
+
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.Task;
-import org.apache.tools.ant.util.Base64Converter;
 import org.apache.tools.ant.taskdefs.optional.net.SetProxy;
+import org.apache.tools.ant.util.Base64Converter;
 
 /**
  * Creates a splash screen. The splash screen is displayed
@@ -51,6 +53,7 @@ public class SplashTask extends Task {
     private String progressRegExp = null;
     private String displayText = null;
 
+    // class instance so we'll never show two splash screens at the same time
     private static SplashScreen splash = null;
 
     /**
@@ -69,6 +72,7 @@ public class SplashTask extends Task {
      * @deprecated since 1.5.x.
      *             Use org.apache.tools.ant.taskdefs.optional.net.SetProxy
      */
+    @Deprecated
     public void setUseproxy(boolean useProxy) {
         this.useProxy = useProxy;
     }
@@ -79,6 +83,7 @@ public class SplashTask extends Task {
      * @deprecated since 1.5.x.
      *             Use org.apache.tools.ant.taskdefs.optional.net.SetProxy
      */
+    @Deprecated
     public void setProxy(String proxy) {
         this.proxy = proxy;
     }
@@ -89,6 +94,7 @@ public class SplashTask extends Task {
      * @deprecated since 1.5.x.
      *             Use org.apache.tools.ant.taskdefs.optional.net.SetProxy
      */
+    @Deprecated
     public void setPort(String port) {
         this.port = port;
     }
@@ -99,6 +105,7 @@ public class SplashTask extends Task {
      * @deprecated since 1.5.x.
      *             Use org.apache.tools.ant.taskdefs.optional.net.SetProxy
      */
+    @Deprecated
     public void setUser(String user) {
         this.user = user;
     }
@@ -109,6 +116,7 @@ public class SplashTask extends Task {
      * @deprecated since 1.5.x.
      *             Use org.apache.tools.ant.taskdefs.optional.net.SetProxy
      */
+    @Deprecated
     public void setPassword(String password) {
         this.password = password;
     }
@@ -116,7 +124,7 @@ public class SplashTask extends Task {
     /**
      * how long to show the splash screen in milliseconds,
      * optional; default 5000 ms.
-     * @param duration the spash duration in milliseconds
+     * @param duration the splash duration in milliseconds
      */
     public void setShowduration(int duration) {
         this.showDuration = duration;
@@ -138,7 +146,7 @@ public class SplashTask extends Task {
 
     /**
      * Sets the display text presented in the splash window.
-     * optional; defaults to "Building ..." 
+     * optional; defaults to "Building ..."
      * @param displayText the display text presented the splash window
      * @since Ant 1.8.0
      */
@@ -150,12 +158,13 @@ public class SplashTask extends Task {
      * Execute the task.
      * @throws BuildException on error
      */
+    @Override
     public void execute() throws BuildException {
         if (splash != null) {
             splash.setVisible(false);
             getProject().removeBuildListener(splash);
             splash.dispose();
-            splash = null;
+            splash = null; //NOSONAR
         }
 
         log("Creating new SplashScreen", Project.MSG_VERBOSE);
@@ -240,7 +249,7 @@ public class SplashTask extends Task {
 
                 try {
                     ImageIcon img = new ImageIcon(bout.toByteArray());
-                    splash = new SplashScreen(img, progressRegExp, displayText);
+                    splash = new SplashScreen(img, progressRegExp, displayText); //NOSONAR
                     success = true;
                 } catch (Throwable e) {
                     logHeadless(e);
@@ -254,13 +263,13 @@ public class SplashTask extends Task {
                     // swallow if there was an error before so that
                     // original error will be passed up
                     if (success) {
-                        throw new BuildException(ioe);
+                        throw new BuildException(ioe); //NOSONAR
                     }
                 }
             }
         } else {
             try {
-                splash = new SplashScreen("Image Unavailable.", progressRegExp,
+                splash = new SplashScreen("Image Unavailable.", progressRegExp, //NOSONAR
                                           displayText);
                 success = true;
             } catch (Throwable e) {
