@@ -21,7 +21,7 @@
  * an NSF grant I should say the following boilerplate:
  *
  * This material is based upon work supported by the National Science
- * Foundaton under Grant No. EIA-0196404. Any opinions, findings, and
+ * Foundation under Grant No. EIA-0196404. Any opinions, findings, and
  * conclusions or recommendations expressed in this material are those
  * of the author and do not necessarily reflect the views of the
  * National Science Foundation.
@@ -29,36 +29,36 @@
 
 package org.apache.tools.ant.taskdefs.optional.unix;
 
-import java.io.File;
-import java.io.IOException;
-import java.io.PrintStream;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
+import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileNotFoundException;
-
-import java.util.Vector;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.PrintStream;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.Hashtable;
+import java.util.Iterator;
 import java.util.Properties;
+import java.util.Vector;
 
-import org.apache.tools.ant.Project;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.DirectoryScanner;
+import org.apache.tools.ant.Project;
 import org.apache.tools.ant.dispatch.DispatchTask;
 import org.apache.tools.ant.dispatch.DispatchUtils;
 import org.apache.tools.ant.taskdefs.Execute;
 import org.apache.tools.ant.taskdefs.LogOutputStream;
 import org.apache.tools.ant.types.FileSet;
-import org.apache.tools.ant.types.Commandline;
 import org.apache.tools.ant.util.FileUtils;
+import org.apache.tools.ant.util.SymbolicLinkUtils;
 
 /**
  * Creates, Deletes, Records and Restores Symlinks.
  *
- * <p> This task performs several related operations. In the most trivial
+ * <p>This task performs several related operations. In the most trivial
  * and default usage, it creates a link specified in the link attribute to
  * a resource specified in the resource attribute. The second usage of this
  * task is to traverse a directory structure specified by a fileset,
@@ -67,33 +67,33 @@ import org.apache.tools.ant.util.FileUtils;
  * directory structure specified by a fileset, looking for properties files
  * (also specified as included in the fileset) and recreate the links
  * that have been previously recorded for each directory. Finally, it can be
- * used to remove a symlink without deleting the associated resource.
+ * used to remove a symlink without deleting the associated resource.</p>
  *
- * <p> Usage examples:
+ * <p>Usage examples:</p>
  *
- * <p> Make a link named &quot;foo&quot; to a resource named
- * &quot;bar.foo&quot; in subdir:
+ * <p>Make a link named &quot;foo&quot; to a resource named
+ * &quot;bar.foo&quot; in subdir:</p>
  * <pre>
  * &lt;symlink link=&quot;${dir.top}/foo&quot; resource=&quot;${dir.top}/subdir/bar.foo&quot;/&gt;
  * </pre>
  *
- * <p> Record all links in subdir and its descendants in files named
- * &quot;dir.links&quot;:
+ * <p>Record all links in subdir and its descendants in files named
+ * &quot;dir.links&quot;:</p>
  * <pre>
  * &lt;symlink action=&quot;record&quot; linkfilename=&quot;dir.links&quot;&gt;
  *    &lt;fileset dir=&quot;${dir.top}&quot; includes=&quot;subdir&#47;**&quot; /&gt;
  * &lt;/symlink&gt;
  * </pre>
  *
- * <p> Recreate the links recorded in the previous example:
+ * <p>Recreate the links recorded in the previous example:</p>
  * <pre>
  * &lt;symlink action=&quot;recreate&quot;&gt;
  *    &lt;fileset dir=&quot;${dir.top}&quot; includes=&quot;subdir&#47;**&#47;dir.links&quot; /&gt;
  * &lt;/symlink&gt;
  * </pre>
  *
- * <p> Delete a link named &quot;foo&quot; to a resource named
- * &quot;bar.foo&quot; in subdir:
+ * <p>Delete a link named &quot;foo&quot; to a resource named
+ * &quot;bar.foo&quot; in subdir:</p>
  * <pre>
  * &lt;symlink action=&quot;delete&quot; link=&quot;${dir.top}/foo&quot;/&gt;
  * </pre>
@@ -107,13 +107,15 @@ import org.apache.tools.ant.util.FileUtils;
  * or action=&quot;recreate&quot;, but action=&quot;record&quot; should still
  * work. Finally, the lack of support for symlinks in Java means that all links
  * are recorded as links to the <strong>canonical</strong> resource name.
- * Therefore the link: <code>link --> subdir/dir/../foo.bar</code> will be
+ * Therefore the link: <code>link --&gt; subdir/dir/../foo.bar</code> will be
  * recorded as <code>link=subdir/foo.bar</code> and restored as
- * <code>link --> subdir/foo.bar</code>.
+ * <code>link --&gt; subdir/foo.bar</code>.</p>
  *
  */
 public class Symlink extends DispatchTask {
     private static final FileUtils FILE_UTILS = FileUtils.getFileUtils();
+    private static final SymbolicLinkUtils SYMLINK_UTILS =
+        SymbolicLinkUtils.getSymbolicLinkUtils();
 
     private String resource;
     private String link;
@@ -127,6 +129,7 @@ public class Symlink extends DispatchTask {
      * Initialize the task.
      * @throws BuildException on error.
      */
+    @Override
     public void init() throws BuildException {
         super.init();
         setDefaults();
@@ -136,6 +139,7 @@ public class Symlink extends DispatchTask {
      * The standard method for executing any task.
      * @throws BuildException on error.
      */
+    @Override
     public synchronized void execute() throws BuildException {
         if (executing) {
             throw new BuildException(
@@ -182,7 +186,9 @@ public class Symlink extends DispatchTask {
                 return;
             }
             log("Removing symlink: " + link);
-            deleteSymlink(link);
+            SYMLINK_UTILS.deleteSymbolicLink(FILE_UTILS
+                                             .resolveFile(new File("."), link),
+                                             this);
         } catch (FileNotFoundException fnfe) {
             handleError(fnfe.toString());
         } catch (IOException ioe) {
@@ -212,11 +218,11 @@ public class Symlink extends DispatchTask {
                 // handle the case where lnk points to a directory (bug 25181)
                 try {
                     File test = new File(lnk);
-                    if (!FILE_UTILS.isSymbolicLink(null, lnk)) {
+                    if (!SYMLINK_UTILS.isSymbolicLink(lnk)) {
                         doLink(res, lnk);
                     } else if (!test.getCanonicalPath().equals(
                         new File(res).getCanonicalPath())) {
-                        deleteSymlink(lnk);
+                        SYMLINK_UTILS.deleteSymbolicLink(test, this);
                         doLink(res, lnk);
                     } // else lnk exists, do nothing
                 } catch (IOException ioe) {
@@ -321,6 +327,7 @@ public class Symlink extends DispatchTask {
      *
      * @param action    The action to perform.
      */
+    @Override
     public void setAction(String action) {
         super.setAction(action);
     }
@@ -371,14 +378,16 @@ public class Symlink extends DispatchTask {
      *
      * @param path    A string containing the path of the symlink to delete.
      *
-     * @throws FileNotFoundException   When the path results in a
-     *                                   <code>File</code> that doesn't exist.
      * @throws IOException             If calls to <code>File.rename</code>
      *                                   or <code>File.delete</code> fail.
+     * @deprecated use
+     * org.apache.tools.ant.util.SymbolicLinkUtils#deleteSymbolicLink
+     * instead
      */
+    @Deprecated
     public static void deleteSymlink(String path)
-        throws IOException, FileNotFoundException {
-        deleteSymlink(new File(path));
+        throws IOException {
+        SYMLINK_UTILS.deleteSymbolicLink(new File(path), null);
     }
 
     /**
@@ -386,58 +395,33 @@ public class Symlink extends DispatchTask {
      *
      * <p>This is a utility method that removes a unix symlink without removing
      * the resource that the symlink points to. If it is accidentally invoked
-     * on a real file, the real file will not be harmed, but an exception
-     * will be thrown when the deletion is attempted. This method works by
+     * on a real file, the real file will not be harmed.</p>
+     *
+     * <p>This method works by
      * getting the canonical path of the link, using the canonical path to
      * rename the resource (breaking the link) and then deleting the link.
      * The resource is then returned to its original name inside a finally
      * block to ensure that the resource is unharmed even in the event of
-     * an exception.
+     * an exception.</p>
+     *
+     * <p>Since Ant 1.8.0 this method will try to delete the File object if
+     * it reports it wouldn't exist (as symlinks pointing nowhere usually do).
+     * Prior version would throw a FileNotFoundException in that case.</p>
      *
      * @param linkfil    A <code>File</code> object of the symlink to delete.
      *
-     * @throws FileNotFoundException   When the path results in a
-     *                                   <code>File</code> that doesn't exist.
      * @throws IOException             If calls to <code>File.rename</code>,
      *                                   <code>File.delete</code> or
      *                                   <code>File.getCanonicalPath</code>
      *                                   fail.
+     * @deprecated use
+     * org.apache.tools.ant.util.SymbolicLinkUtils#deleteSymbolicLink
+     * instead
      */
+    @Deprecated
     public static void deleteSymlink(File linkfil)
-        throws IOException, FileNotFoundException {
-        if (!linkfil.exists()) {
-            throw new FileNotFoundException("No such symlink: " + linkfil);
-        }
-        // find the resource of the existing link:
-        File canfil = linkfil.getCanonicalFile();
-
-        // rename the resource, thus breaking the link:
-        File temp = FILE_UTILS.createTempFile("symlink", ".tmp",
-                                              canfil.getParentFile(), false, true);
-        try {
-            try {
-                FILE_UTILS.rename(canfil, temp);
-            } catch (IOException e) {
-                throw new IOException(
-                    "Couldn't rename resource when attempting to delete "
-                    + linkfil);
-            }
-            // delete the (now) broken link:
-            if (!linkfil.delete()) {
-                throw new IOException("Couldn't delete symlink: " + linkfil
-                    + " (was it a real file? is this not a UNIX system?)");
-            }
-        } finally {
-            // return the resource to its original name:
-            try {
-                FILE_UTILS.rename(temp, canfil);
-            } catch (IOException e) {
-                throw new IOException("Couldn't return resource " + temp
-                    + " to its original name: " + canfil.getAbsolutePath()
-                    + "\n THE RESOURCE'S NAME ON DISK HAS "
-                    + "BEEN CHANGED BY THIS ERROR!\n");
-            }
-        }
+        throws IOException {
+        SYMLINK_UTILS.deleteSymbolicLink(linkfil, null);
     }
 
     /**
@@ -446,6 +430,7 @@ public class Symlink extends DispatchTask {
      *
      * @param properties     The properties object to be written.
      * @param dir            The directory for which we are writing the links.
+     * @throws BuildException if the property file could not be written
      */
     private void writePropertyFile(Properties properties, File dir)
         throws BuildException {
@@ -466,6 +451,7 @@ public class Symlink extends DispatchTask {
      *
      * @param msg    The message to log, or include in the
      *                  <code>BuildException</code>.
+     * @throws BuildException with the message if failonerror=true
      */
     private void handleError(String msg) {
         if (failonerror) {
@@ -477,33 +463,47 @@ public class Symlink extends DispatchTask {
     /**
      * Conduct the actual construction of a link.
      *
-     * <p> The link is constructed by calling <code>Execute.runCommand</code>.
+     * <p>The link is constructed by calling <code>Execute.runCommand</code>.</p>
      *
      * @param res   The path of the resource we are linking to.
      * @param lnk       The name of the link we wish to make.
+     * @throws BuildException when things go wrong
      */
     private void doLink(String res, String lnk) throws BuildException {
         File linkfil = new File(lnk);
-        if (overwrite && linkfil.exists()) {
-            try {
-                deleteSymlink(linkfil);
-            } catch (FileNotFoundException fnfe) {
-                handleError("Symlink disappeared before it was deleted: " + lnk);
-            } catch (IOException ioe) {
-                handleError("Unable to overwrite preexisting link: " + lnk);
+        String options = "-s";
+        if (overwrite) {
+            options += "f";
+            if (linkfil.exists()) {
+                try {
+                    SYMLINK_UTILS.deleteSymbolicLink(linkfil, this);
+                } catch (FileNotFoundException fnfe) {
+                    log("Symlink disappeared before it was deleted: " + lnk);
+                } catch (IOException ioe) {
+                    log("Unable to overwrite preexisting link or file: " + lnk,
+                        ioe, Project.MSG_INFO);
+                }
+            }
+        }
+        String[] cmd = new String[] {"ln", options, res, lnk};
+        try {
+            Execute.runCommand(this, cmd);
+        } catch (BuildException failedToExecute) {
+            if (failonerror) {
+                throw failedToExecute;
+            } else {
+                //log at the info level, and keep going.
+                log(failedToExecute.getMessage(), failedToExecute, Project.MSG_INFO);
             }
         }
-        String[] cmd = new String[] {"ln", "-s", res, lnk};
-        log(Commandline.toString(cmd));
-        Execute.runCommand(this, cmd);
     }
 
     /**
      * Find all the links in all supplied filesets.
      *
-     * <p> This method is invoked when the action attribute is
+     * <p>This method is invoked when the action attribute is
      * &quot;record&quot;. This means that filesets are interpreted
-     * as the directories in which links may be found.
+     * as the directories in which links may be found.</p>
      *
      * @param v   The filesets specified by the user.
      * @return A HashSet of <code>File</code> objects containing the
@@ -511,7 +511,8 @@ public class Symlink extends DispatchTask {
      */
     private HashSet findLinks(Vector v) {
         HashSet result = new HashSet();
-        for (int i = 0; i < v.size(); i++) {
+        final int size = v.size();
+        for (int i = 0; i < size; i++) {
             FileSet fs = (FileSet) v.get(i);
             DirectoryScanner ds = fs.getDirectoryScanner(getProject());
             String[][] fnd = new String[][]
@@ -523,7 +524,7 @@ public class Symlink extends DispatchTask {
                         File f = new File(dir, fnd[j][k]);
                         File pf = f.getParentFile();
                         String name = f.getName();
-                        if (FILE_UTILS.isSymbolicLink(pf, name)) {
+                        if (SYMLINK_UTILS.isSymbolicLink(pf, name)) {
                             result.add(new File(pf.getCanonicalFile(), name));
                         }
                     } catch (IOException e) {
@@ -538,10 +539,10 @@ public class Symlink extends DispatchTask {
     /**
      * Load links from properties files included in one or more FileSets.
      *
-     * <p> This method is only invoked when the action attribute is set to
+     * <p>This method is only invoked when the action attribute is set to
      * &quot;recreate&quot;. The filesets passed in are assumed to specify the
      * names of the property files with the link information and the
-     * subdirectories in which to look for them.
+     * subdirectories in which to look for them.</p>
      *
      * @param v    The <code>FileSet</code>s for this task.
      * @return            The links to be made.
@@ -549,7 +550,8 @@ public class Symlink extends DispatchTask {
     private Properties loadLinks(Vector v) {
         Properties finalList = new Properties();
         // loop through the supplied file sets:
-        for (int i = 0; i < v.size(); i++) {
+        final int size = v.size();
+        for (int i = 0; i < size; i++) {
             FileSet fs = (FileSet) v.elementAt(i);
             DirectoryScanner ds = new DirectoryScanner();
             fs.setupDirectoryScanner(ds, getProject());
@@ -563,8 +565,10 @@ public class Symlink extends DispatchTask {
                 File inc = new File(dir, incs[j]);
                 File pf = inc.getParentFile();
                 Properties lnks = new Properties();
+                InputStream is = null;
                 try {
-                    lnks.load(new BufferedInputStream(new FileInputStream(inc)));
+                    is = new BufferedInputStream(new FileInputStream(inc));
+                    lnks.load(is);
                     pf = pf.getCanonicalFile();
                 } catch (FileNotFoundException fnfe) {
                     handleError("Unable to find " + incs[j] + "; skipping it.");
@@ -573,6 +577,8 @@ public class Symlink extends DispatchTask {
                     handleError("Unable to open " + incs[j]
                                 + " or its parent dir; skipping it.");
                     continue;
+                } finally {
+                    FileUtils.close(is);
                 }
                 lnks.list(new PrintStream(
                     new LogOutputStream(this, Project.MSG_INFO)));
